
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "csound.hpp"
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <cstdlib>
#include <iomanip>
#include <cstring>

extern "C" {
	#include "lua.h"
	#include "lualib.h"
	#include "lauxlib.h"
}

using namespace std;
char term;
lua_State* L;
void report_errors(lua_State *L, int status);
void luaFunction();
static float getnum(FILE* inf, char *term);

// see http://csl.sublevel3.org/lua/
void report_errors(lua_State *L, int status)
{
  if ( status!=0 ) {
    std::cerr << "-- " << lua_tostring(L, -1) << std::endl;
    lua_pop(L, 1); // remove error message
  }
}

void luaFunction ()
{
	int status;
	L = luaL_newstate();
	luaL_openlibs(L);

        /* Load the file containing the script we are going to run */
    status = luaL_loadfile(L, "fileManip.lua");

    if (status) {
        fprintf(stderr, "Couldn't load file: %s\n", lua_tostring(L, -1));
        exit(1);
    }

    std::cerr << "-- Loading file: " << "fileManip.lua" << std::endl;

    int ss = luaL_dofile(L,"fileManip.lua");

    if ( ss==0 ) {
      // execute Lua program
      ss = lua_pcall(L, 0, LUA_MULTRET, 0);
    }

    report_errors(L, ss);
    lua_close(L);
    std::cerr << std::endl;

}

//parser code by John ffitch from pv_import.c utility
 static float getnum(FILE* inf, char *term)
{
    char buff[100];
    int  cc;
    int p = 0;
    while ((cc=getc(inf))!=',' && cc!='\n' && cc!=EOF) buff[p++] = cc;
    buff[p]='\0';
    *term = cc;
    return (float)atof(buff);
}

int main(int argc, char *argv[])
{
Csound *csound;
int c, j, k,res, result;
csound = new Csound;
PVSDATEXT datain;
FILE *inf;
memset(&datain, sizeof(PVSDATEXT), 0);

luaFunction();

    datain.N = 1024;
    datain.sliding = 0;
    datain.NB = 0;
    datain.overlap = 256;
    datain.winsize = 1024;
    datain.wintype = 1;
    datain.format = 0;
    datain.framecount = 0;
    datain.frame = (float *) calloc(sizeof(float),(datain.N+2));

csound->PreCompile();
result = csound->Compile ("pvsintest.csd");
//++++++++++
//INPUT from FILE
inf = fopen("Erhu2.txt", "r");
c = getc (inf);
if (inf == NULL) {
      printf("Cannot open input file %s\n");
      return 1;
    }

//while (!feof(inf))
while (c != EOF)
{
    while (!result)
    {
 	for(k=0; k < 1026; k++)
        //alternate amp and freq frames
	if (k % 2 == 0){
	  datain.frame[k]=getnum(inf, &term);
        }
        else {
          datain.frame[k]=getnum(inf, &term);
        }

      //send in the pvs in bus data
      csound->PvsinSet(&datain, 1);
      // one ksmps pass
      result = csound->PerformKsmps();
      } //ends inner while
      goto ending;
} //ends outer while

ending:
     fclose(inf);
    //delete Csound instance
    delete csound;
    //free memory
    free(datain.frame);
    return (result >= 0 ? 0 : result);
} //end main

